﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Sparkle.Effects.TextEffects;
using Sparkle.Effects.TextEffects.Simple;
namespace Sparkle.Effects.Renderer
{
    public class TextEffectRenderer : GameComponent
    {
        private Game _game;
        private ContentManager Content;

       //Flags indicating if the renderer has been initialized and loaded
        internal bool IsInitialized=false;
        internal bool IsContentLoaded=false;

        /// <summary>
        /// List with all created effects
        /// </summary>
        internal List<TextEffectObject> TextEffects = new List<TextEffectObject>();
        
        /// <summary>
        /// Debug font
        /// </summary>
        internal static SpriteFont debugFont;

        /// <summary>
        /// Standard text effect font
        /// </summary>
        internal static SpriteFont StandardFont;
        
        public TextEffectRenderer(Game game)
            : base(game)
        {
            _game = game;
            Content = new ResourceContentManager(_game.Services, SparkleContent.ResourceManager);
        }

        /// <summary>
        /// Initialize the TextEffectRenderer
        /// </summary>
        public override void Initialize()
        {
            IsInitialized = true;
            base.Initialize();
        }
        /// <summary>
        /// Load the renderers internal content
        /// </summary>
        public void LoadContent()
        {
            TextWriterEffect.Marker = Content.Load<Texture2D>("Marker");
            debugFont = Content.Load<SpriteFont>("debugFont");
            StandardFont = Content.Load<SpriteFont>("StandardFont");
            IsContentLoaded = true;
        }
        /// <summary>
        /// Adds an effect to the renderer
        /// </summary>
        /// <param name="textEffectObject">Effect to add</param>
        internal void AddEffect(TextEffectObject textEffectObject)
        {
            if (!IsInitialized)
                throw new EngineException("The TextEffectRenderer has not been initialized!");
            if (!IsContentLoaded)
                throw new EngineException("The TextEffectRenderer has not loaded its content!");

            Console.WriteLine("Adding effect to renderer");
            textEffectObject.Font = StandardFont;
            TextEffects.Add(textEffectObject);
        }

        public override void Update(GameTime gameTime)
        {
            for (int i = 0; i < TextEffects.Count; i++)
            {
                if (TextEffects[i].TransitionState == TextEffectObject.TextEffectState.Finished)
                {
                    TextEffects[i].EffectFinishedEvent();
                    TextEffects.Remove(TextEffects[i]);
                    if (TextEffects.Count <= 0)
                        GC.Collect();
                }
                else
                    TextEffects[i].Update(gameTime);
            }
            base.Update(gameTime);
        }
        public void Draw(GameTime gameTime, SpriteBatch sp)
        {
            sp.Begin();
            for (int i = 0; i < TextEffects.Count; i++)
            {
                TextEffects[i].Draw(gameTime, sp);
            }
            float frameRate = 1 / (float)gameTime.ElapsedGameTime.TotalSeconds;
            //sp.DrawString(debugFont, "Active Effects:; " + TextEffects.Count, Vector2.Zero, Color.Magenta);
            //sp.DrawString(debugFont, "Current framerate: " + frameRate, new Vector2(0,10), Color.Magenta);
            sp.End();
        }

        /// <summary>
        /// Terminates all active effects
        /// </summary>
        public void TerminateAllEffects()
        {
            foreach (TextEffectObject o in TextEffects)
                o.TransitionState = TextEffectObject.TextEffectState.Finished;
        }



        
    }
}
