﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Sparkle.Effects.Renderer;
namespace Sparkle.Effects.TextEffects.Simple
{
    public class SlideTextEffect : TextEffectObject
    {
        //Slide end position
        public Vector2 EndPosition;
        //Slide by letter block or the complete word
        public bool SlideByBlock = false;
        //Start as transparent
        public bool StartTransparent = false;
        //Fade out by letter or by complete word
        public bool FadeOutByBlock = false;
        //Time for the effect to pause until it starts to fade
        public float PauseUntilFade 
        { set { PauseTime = value; } }
        //Time in milliseconds for the effect to slide in
        public float SlideInTimer
        {
            set { PlayEffectDuration = value; }
        }

        private Vector2 effectStartPosition;
        private Vector2 Direction = new Vector2(1, 1);
        private Vector2 BlockOffset;
        public SlideTextEffect(TextEffectRenderer renderer)
            : base(renderer)
        { 
        
        }
        public override void InitializeEffect(string Text)
        {
            base.Initialize(Text,StartTransparent);
            effectStartPosition = Position;
            if (SlideByBlock)
            {
                //Direction Right,Down
                if (EndPosition.X< effectStartPosition.X)
                    //Direction Left,Down
                    Direction.X *= -1;
                if (EndPosition.Y < effectStartPosition.Y)
                    //Direction Left or Right, Up
                    Direction.Y *= -1;
                if (EndPosition.Y == effectStartPosition.Y)
                    Direction.Y *= 0;
                if (EndPosition.X == effectStartPosition.X)
                    Direction.X *= 0;
                BlockOffset = new Vector2(Math.Abs(effectStartPosition.X - EndPosition.X), Math.Abs(effectStartPosition.Y - EndPosition.Y));
                BlockOffset = Vector2.Multiply(BlockOffset, Direction);
                BlockOffset = BlockOffset * -1;
             
                for (int i = 0; i < Objects.Count; i++)
                {
                    Objects[i].BlockPosition = new Vector2(BlockOffset.X + Objects[i].BlockPosition.X, BlockOffset.Y);
                }
                BlockOffset = BlockOffset * -1;
                Position = EndPosition;
            }
            
        }

        internal override void PlayTransitionOnEffect(float percentElapsed, GameTime gt)
        {
            
        }
        internal override void PlayEffect(float percentElapsed, GameTime gt)
        {
           
            if (SlideByBlock)
            {
              
                if (BlockIndex < Objects.Count)
                {
                    if (StartTransparent)
                        Objects[BlockIndex].BlockColor = Objects[BlockIndex].BlockOriginalColor * percentElapsed;
                    
                    float x = MathHelper.SmoothStep(Objects[BlockIndex].BlockPosition.X,Objects[BlockIndex].PositionInWord_X, percentElapsed);
                    float Y = MathHelper.SmoothStep(Objects[BlockIndex].BlockPosition.Y, 0, percentElapsed);
                    
                    Objects[BlockIndex].BlockPosition = new Vector2(x,Y);
                    if (percentElapsed >= 1)
                    {
                        BlockIndex++;
                        Elapsed_PlayEffectDuration = 0;
                    }
                    
                }
            }
            else
            {
                if (StartTransparent)
                    for (int i = BlockIndex; i < Objects.Count; i++)
                        Objects[i].BlockColor = Objects[i].BlockOriginalColor * percentElapsed;
                Position = new Vector2(
                          MathHelper.SmoothStep(effectStartPosition.X, EndPosition.X, percentElapsed),
                          MathHelper.SmoothStep(effectStartPosition.Y, EndPosition.Y, percentElapsed));
            }
            
        }

        internal override void PlayTransitionOffEffect(float percentElapsed, GameTime gt)
        {
            base.FadeOutEffect(percentElapsed, FadeOutByBlock);
        }
    }
}
