﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Sparkle.Effects.Renderer;
namespace Sparkle.Effects.TextEffects.Simple
{
    public class TextWriterEffect : TextEffectObject
    {

        public static Texture2D Marker { get; set; }
        public bool FadeOutByBlock = false;
        private Vector2 MarkerPosition;
        private Vector2 MarkerScale;
        private Color MarkerColor = Color.White;
        private Color MarkerBaseColor;
        private float MarkerColorAlpha;
        private float MarkerWidht = 0;
        
        /// <summary>
        /// The duration of the effect to type the text
        /// </summary>
        public float TypeTextTimer
        {
            set { PlayEffectDuration = value; }
        }
        public TextWriterEffect(TextEffectRenderer renderer)
            : base(renderer)
        {
            MarkerBaseColor = MarkerColor;
        }
        /// <summary>
        /// Set up the effect with the provided Text
        /// </summary>
        /// <param name="Text">Text to display</param>
        public override void InitializeEffect(string Text)
        {
            base.Initialize(Text,true);
            for (int i = 0; i < Objects.Count; i++)
            {
                if (Objects[i].BlockSize.X > MarkerWidht)
                    MarkerWidht = Objects[i].BlockSize.X;
            }
            MarkerScale = new Vector2(MarkerWidht / Marker.Width, (Objects[BlockIndex].BlockSize.Y / Marker.Height));
            MarkerColor = MarkerColor * 0;
        }
      
        internal override void PlayTransitionOnEffect(float percentElapsed, GameTime gt)
        {
           
        }
        
        internal override void PlayEffect(float percentElapsed, GameTime gt)
        {
            if (BlockIndex < Objects.Count)
            {
                MarkerPosition =Vector2.Add(Position, Objects[BlockIndex].BlockPosition);
                if (percentElapsed >= 1)
                {
                    Objects[BlockIndex].BlockColor = Objects[BlockIndex].BlockOriginalColor;
                    BlockIndex++;
                    Elapsed_PlayEffectDuration = 0;
                }
            }
            else
            {
                MarkerPosition = new Vector2(Objects[Objects.Count - 1].BlockSize.X + Objects[Objects.Count - 1].BlockPosition.X + Position.X, MarkerPosition.Y);
            }
            MarkerColorAlpha += (float)gt.ElapsedGameTime.TotalMilliseconds / 40;
            MarkerColor = MarkerBaseColor * (float)Math.Sin(MarkerColorAlpha);
        }
        
        internal override void PlayTransitionOffEffect(float percentElapsed, GameTime gt)
        {
            if (FadeOutTimer != -1)
            {
                MarkerColor = MarkerColor * (1 - percentElapsed * 2);
                base.FadeOutEffect(percentElapsed, FadeOutByBlock);
            }
            else
            {
                MarkerColorAlpha += (float)gt.ElapsedGameTime.TotalMilliseconds / 40;
                MarkerColor = MarkerBaseColor * (float)Math.Sin(MarkerColorAlpha);
            }
        }
        
        protected internal override void Draw(GameTime gt, SpriteBatch sp)
        {
            
            base.Draw(gt, sp);
            sp.Draw(Marker, MarkerPosition, null, MarkerColor, 0f, Vector2.Zero, MarkerScale, SpriteEffects.None, 1f);
        }
       
    }
}
